@@ -1,7 +1,7 @@
 
 /*
  *  expState.c
- *  $Id: expState.c,v 1.48 2004/02/24 02:47:52 bkorb Exp $
+ *  $Id: expState.c,v 1.53 2004/08/15 17:01:34 bkorb Exp $
  *  This module implements expression functions that
  *  query and get state information from AutoGen data.
  */
@@ -541,10 +541,14 @@ ag_scm_tpl_file( SCM full )
  *
  * exparg: msg-fmt, formatting for line message, optional
  *
- * doc: Returns the file and line number of the current template macro
- *      using either the default format, "from %s line %d", or else
- *      the format you supply.  For example, if you want to insert a "C"
- *      language file-line directive, you would supply the format "# %2$d %1$s".
+ * doc:
+ *
+ *  Returns the file and line number of the current template macro using either
+ *  the default format, "from %s line %d", or else the format you supply.
+ *  For example, if you want to insert a "C" language file-line directive,
+ *  you would supply the format "# %2$d %1$s".  With the current version of
+ *  AutoGen, it is even safe to use the formatting string, "%2$d".  AutoGen
+ *  uses an argument vector version of printf:  @xref{snprintfv}.
 =*/
 SCM
 ag_scm_tpl_file_line( SCM fmt )
@@ -565,7 +569,12 @@ ag_scm_tpl_file_line( SCM fmt )
         else pz = zScribble;
     }
 
-    sprintf( pz, pzFmt, pCurTemplate->pzFileName, pCurMacro->lineNo );
+    {
+        void* args[2];
+        args[0] = (void*)pCurTemplate->pzFileName;
+        args[1] = (void*)pCurMacro->lineNo;
+        sprintfv( pz, pzFmt, (snv_constpointer*)args  );
+    }
 
     {
         SCM res = gh_str02scm( pz );
@@ -575,8 +584,6 @@ ag_scm_tpl_file_line( SCM fmt )
         return res;
     }
 }
-
-#include "expr.ini"
 /*
  * Local Variables:
  * mode: C
