@@ -1,12 +1,12 @@
 
 /*
- *  $Id: funcEval.c,v 1.39 2002/03/19 04:14:26 bkorb Exp $
+ *  $Id: funcEval.c,v 1.54 2004/10/11 23:30:00 bkorb Exp $
  *
  *  This module evaluates macro expressions.
  */
 
 /*
- *  AutoGen copyright 1992-2002 Bruce Korb
+ *  AutoGen copyright 1992-2004 Bruce Korb
  *
  *  AutoGen is free software.
  *  You may redistribute it and/or modify it under the terms of the
@@ -24,25 +24,14 @@
  *             59 Temple Place - Suite 330,
  *             Boston,  MA  02111-1307, USA.
  */
-#ifndef DEFINE_LOAD_FUNCTIONS
-
-#include "expr.h"
-#include "autogen.h"
-
-#include <compat/compat.h>
-#include REGEX_HEADER
-
-tSCC        zNil[] = "";
 
 STATIC int exprType( char* pz );
 
-
 EXPORT char*
 resolveSCM( SCM s )
 {
-    static char z[32];
+    static char z[48];
     char*  pzRes = z;
-    int    len;
 
     switch (gh_type_e( s )) {
     case GH_TYPE_BOOLEAN:         
@@ -64,16 +53,21 @@ resolveSCM( SCM s )
         pzRes = "** Pair **"; break;
 
     case GH_TYPE_NUMBER:
-        snprintf( z, sizeof(z), "%d", gh_scm2ulong(s) ); break;
+        snprintf( z, sizeof(z), "%ld", gh_scm2ulong(s) ); break;
 
     case GH_TYPE_PROCEDURE:
 #ifdef SCM_SUBR_ENTRY
+    {
+        void* x = &SCM_SUBR_ENTRY(s);
+
         snprintf( z, sizeof(z), "** Procedure 0x%08lX **",
-                  SCM_SUBR_ENTRY(s) );
+                  (unsigned long)x );
+        break;
+    }
 #else
         pzRes = "** PROCEDURE **";
-#endif
         break;
+#endif
 
     case GH_TYPE_LIST:
         pzRes = "** LIST **"; break;
@@ -152,13 +146,14 @@ evalExpression( ag_bool* pMustFree )
                 /*
                  *  Emit only if found
                  */
-                return (char*)zNil;
+                return (char*)zDefaultNil;
 
             case (EMIT_IF_ABSENT | EMIT_ALWAYS):
                 /*
                  *  Emit inconsistently :-}
                  */
                 AG_ABEND_IN( pT, pMac, "PROGRAM ERROR:  ambiguous expr code" );
+                /* NOTREACHED */
             }
         }
 
@@ -169,13 +164,13 @@ evalExpression( ag_bool* pMustFree )
             tSCC zBlock[] = "attempted to use block macro in eval expression";
 
             if ((code & EMIT_IF_ABSENT) != 0)
-                return (char*)zNil;
+                return (char*)zDefaultNil;
 
             if (  (pDef->valType != VALTYP_TEXT)
                && ((code & EMIT_PRIMARY_TYPE) == EMIT_VALUE)  ) {
                 fprintf( pfTrace, zTplWarn, pT->pzFileName, pMac->lineNo,
                          zBlock );
-                return (char*)zNil;
+                return (char*)zDefaultNil;
             }
 
             /*
@@ -196,12 +191,11 @@ evalExpression( ag_bool* pMustFree )
                 if (pDef->valType != VALTYP_TEXT) {
                     fprintf( pfTrace, zTplWarn, pT->pzFileName, pMac->lineNo,
                              zBlock );
-                    return (char*)zNil;
+                    return (char*)zDefaultNil;
                 }
 
                 *pMustFree = AG_TRUE;
-                pzText = asprintf( pT->pzTemplText + pMac->ozText,
-                                   pDef->pzValue );
+                pzText = aprf(pT->pzTemplText + pMac->ozText, pDef->val.pzText);
             }
 
             else if (pMac->ozText != 0)
@@ -214,10 +208,10 @@ evalExpression( ag_bool* pMustFree )
                 if (pDef->valType != VALTYP_TEXT) {
                     fprintf( pfTrace, zTplWarn, pT->pzFileName, pMac->lineNo,
                              zBlock );
-                    return (char*)zNil;
+                    return (char*)zDefaultNil;
                 }
 
-                pzText = pDef->pzValue;
+                pzText = pDef->val.pzText;
             }
 
             code &= EMIT_PRIMARY_TYPE;
@@ -234,12 +228,12 @@ evalExpression( ag_bool* pMustFree )
             *pMustFree = AG_FALSE;
         }
 
-        pzText = pDef->pzValue;
+        pzText = pDef->val.pzText;
         break;
 
     case EMIT_EXPRESSION:
     {
-        SCM res = gh_eval_str( pzText );
+        SCM res = ag_eval( pzText );
 
         if (*pMustFree) {
             AGFREE( (void*)pzText );
@@ -285,12 +279,59 @@ evalExpression( ag_bool* pMustFree )
  *       that triggered the evaluation error.  You should not need to
  *       invoke this routine directly.  Guile will do it automatically.
 =*/
-    SCM 
+SCM 
 ag_scm_error_source_line( void )
 {
-    fprintf( stderr, "\nin %s line %d:  %s\n", pCurTemplate->pzTplName,
-             pCurMacro->lineNo,
+    tSCC zErr[] = "\nGuile/Scheme evaluation error in %s line %d:  %s\n";
+    fprintf( stderr, zErr, pCurTemplate->pzTplName, pCurMacro->lineNo,
              pCurTemplate->pzTemplText + pCurMacro->ozText );
+    fflush( stderr );
+
+    return SCM_UNDEFINED;
+}
+
+
+/*=gfunc emit
+ *
+ * what: emit the text for each argument
+ * general_use:
+ *
+ * exparg: alist, list of arguments to stringify and emit, , list
+ *
+ * doc:  Walk the tree of arguments, displaying the values of displayable
+ *       SCM types.
+=*/
+SCM 
+ag_scm_emit( SCM val )
+{
+    for (;;) {
+        if (val == SCM_UNDEFINED)
+            break;
+
+        if (SCM_NULLP( val ))
+            break;
+
+        if (gh_string_p( val )) {
+            fputs( (char*)ag_scm2zchars( val, "emit val" ), pCurFp->pFile );
+            fflush( pCurFp->pFile );
+            break;
+        }
+
+        switch (gh_type_e( val )) {
+        case GH_TYPE_LIST:
+        case GH_TYPE_PAIR:
+            ag_scm_emit( SCM_CAR( val ));
+            val = SCM_CDR( val );
+            continue;
+
+        default:
+            fputs( resolveSCM( val ), pCurFp->pFile );
+            fflush( pCurFp->pFile );
+            break;
+        }
+
+        break;
+    }
 
     return SCM_UNDEFINED;
 }
@@ -317,7 +358,7 @@ eval( const char* pzExpr )
     switch (*pzExpr) {
     case '(':
     case ';':
-        res = gh_eval_str( (char*)pzExpr );
+        res = ag_eval( (char*)pzExpr );
         break;
 
     case '`':
@@ -350,18 +391,7 @@ eval( const char* pzExpr )
 /*=macfunc EXPR
  *
  *  what:  Evaluate and emit an Expression
- *  alias:  -
- *  alias:  ?
- *  alias:  %
- *
- *  alias:  ;
- *  alias:  (
- *  alias:  `
- *
- *  alias:  '"'
- *  alias:  "'"
- *
- *  handler_proc:
+ *  alias: + - + ? + % + ; + ( + . + '`' + '"' + "'" +
  *
  *  desc:
  *   This macro does not have a name to cause it to be invoked
@@ -370,7 +400,7 @@ eval( const char* pzExpr )
  *   macro is inferred.  The result of the expression evaluation
  *   (@pxref{expression syntax}) is written to the current output.
 =*/
-    tMacro*
+tMacro*
 mFunc_Expr( tTemplate* pT, tMacro* pMac )
 {
     ag_bool needFree;
@@ -383,7 +413,6 @@ mFunc_Expr( tTemplate* pT, tMacro* pMac )
 
     return pMac + 1;
 }
-#endif /* DEFINE_LOAD_FUNCTIONS */
 
 
 STATIC int
@@ -412,7 +441,7 @@ exprType( char* pz )
 /*
  *  mLoad_Expression
  */
-    tMacro*
+tMacro*
 mLoad_Expr( tTemplate* pT, tMacro* pMac, tCC** ppzScan )
 {
     char*    pzCopy; /* next text dest   */
@@ -527,7 +556,9 @@ mLoad_Expr( tTemplate* pT, tMacro* pMac, tCC** ppzScan )
 }
 /*
  * Local Variables:
+ * mode: C
  * c-file-style: "stroustrup"
+ * tab-width: 4
  * indent-tabs-mode: nil
  * End:
- * end of funcEval.c */
+ * end of agen5/funcEval.c */
